@@ -75,11 +75,13 @@
   (if-not (string? header)
     true
     (let [[prefix suffix] (.split candidate "/")
+          superwildcard   "*/*"
           wildcard        (str prefix "/*")
           types           (->> (s/split header #",")
                                (map #(.trim %))
                                (set))]
-      (or (types wildcard)
+      (or (types superwildcard)
+          (types wildcard)
           (types candidate)))))
 
 (defn must-accept-type
@@ -104,8 +106,11 @@
          (json/generate-string {:date-format "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'" :pretty true})
          (rr/response)
          (rr/header "Content-Type" "application/json")
+         (rr/charset "utf-8")
          (rr/status code))))
 
+(def json-response-content-type "application/json; charset=utf-8")
+
 (defn error-response
   "Returns a Ring response object with the status code specified by `code`.
    If `error` is a Throwable, its message is used as the body of the response.
@@ -126,3 +131,26 @@
   (from a `//`, for example) are elided from the result"
   [uri]
   (remove #{""} (.split uri "/")))
+
+(defn leading-uris
+  "Given a URI, return a sequence of all the leading components of
+  that URI.
+
+  Example:
+  (leading-uris \"/foo/bar/baz\")
+  => [\"/foo\", \"/foo/bar\", \"/foo/bar/baz\"]
+
+  (leading-uris \"/foo/bar/baz\" \"|\")
+  => [\"|foo\", \"|foo|bar\", \"|foo|bar|baz\"]
+"
+  ([uri]
+     (leading-uris uri "/"))
+  ([uri delimiter]
+     {:pre  [(.startsWith uri "/")]
+      :post [(coll? %)]}
+     (let [segments (uri-segments uri)
+           f        (fn [[segs strs] u]
+                      (let [segs' (conj segs u)]
+                        [segs'
+                         (conj strs (str delimiter (s/join delimiter segs')))]))]
+       (second (reduce f [[] []] segments)))))
