@@ -18,13 +18,14 @@
             [clojure.tools.cli :as cli]
             [digest]
             [fs.core :as fs])
-  (:use [clojure.core.incubator :only (-?> -?>>)]
-        [clojure.java.io :only (reader)]
+  (:use [clojure.java.io :only (reader)]
         [clojure.set :only (difference union)]
+        [clojure.string :only (split)]
         [clojure.stacktrace :only (print-cause-trace)]
         [clj-time.core :only [now]]
         [clj-time.coerce :only [ICoerce to-date-time]]
         [clj-time.format :only [formatters unparse]]
+        [metrics.timers :only [time! timer]]
         [slingshot.slingshot :only (try+ throw+)]))
 
 ;; ## Type checking
@@ -32,9 +33,9 @@
 (defn array?
   "Returns true if `x` is an array"
   [x]
-  (-?> x
-    (class)
-    (.isArray)))
+  (some-> x
+          (class)
+          (.isArray)))
 
 (defn datetime?
   "Predicate returning whether or not the supplied object is
@@ -44,6 +45,17 @@
     (satisfies? ICoerce x)
     (to-date-time x)))
 
+(defn boolean?
+  "Returns true if the value is a boolean"
+  [value]
+  (instance? Boolean value))
+
+(defn regexp?
+  "Returns true if the type is a regexp pattern"
+  [regexp]
+  {:post [(boolean? %)]}
+  (instance? java.util.regex.Pattern regexp))
+
 ;; ## I/O
 
 (defn lines
@@ -166,7 +178,6 @@
          (recur ret (first ks) (next ks))
          ret))))
 
-
 (defn keyset
   "Returns the set of keys from the supplied map"
   [m]
@@ -185,6 +196,27 @@
   "Returns the sequence of values from the map for the entries with the specified keys"
   (comp vals select-keys))
 
+(defn missing?
+  "Inverse of contains? that supports multiple keys. Will return true if all items are
+  missing from the collection, false otherwise.
+
+  Example:
+
+      ;; Returns true, as :z :f :h are all missing
+      (missing? {:a 'a' :b 'b' :c 'c'} :z :f :h)
+
+      ;; Returns false, as :a is in the collection
+      (missing? {:a 'a' :b 'b' :c 'c'} :z :b)"
+  [coll & keys]
+  {:pre  [(coll? coll)]
+   :post [(boolean? %)]}
+  (reduce (fn [_ key]
+            (if (contains? coll key)
+              (reduced false)
+              true))
+          nil
+          keys))
+
 ;; ## Date and Time
 
 (defn timestamp
@@ -476,13 +508,13 @@
       nil"
   [dn]
   {:pre [(string? dn)]}
-  (-?>> dn
-        (LdapName.)
-        (.getRdns)
-        (filter #(= "CN" (.getType %)))
-        (first)
-        (.getValue)
-        (str)))
+  (some->> dn
+           (LdapName.)
+           (.getRdns)
+           (filter #(= "CN" (.getType %)))
+           (first)
+           (.getValue)
+           (str)))
 
 (defn cn-for-cert
   "Extract the CN from the DN of an x509 certificate. See `cn-for-dn` for details
@@ -517,17 +549,14 @@
 
 ;; ## Hashing
 
-;; This method lookup is surprisingly expensive (on the order of 10x slower),
-;; so we pay the cost once, and define our own digest in terms of it.
-(let [digest-func (get-method digest/digest :default)]
-  (defn utf8-string->sha1
-    "Compute a SHA-1 hash for the UTF-8 encoded version of the supplied
-    string"
-    [s]
-    {:pre  [(string? s)]
-     :post [(string? %)]}
-    (let [bytes (.getBytes s "UTF-8")]
-      (digest-func "sha-1" [bytes]))))
+(defn utf8-string->sha1
+  "Compute a SHA-1 hash for the UTF-8 encoded version of the supplied
+  string"
+  [s]
+  {:pre  [(string? s)]
+   :post [(string? %)]}
+  (let [bytes (.getBytes s "UTF-8")]
+    (digest/sha-1 [bytes])))
 
 (defn bounded-memoize
   "Similar to memoize, but the cache will be reset if the number of entries
@@ -560,4 +589,61 @@
   {:post [(pos? %)]}
   (.availableProcessors (Runtime/getRuntime)))
 
+;; Comparison of JVM versions
 
+(defn compare-jvm-versions
+  "Same behavior as `compare`, but specifically for JVM version
+   strings.  Because Java versions don't follow semver or anything, we
+   need to do some massaging of the input first:
+
+  http://www.oracle.com/technetwork/java/javase/versioning-naming-139433.html"
+  [a b]
+  {:pre  [(string? a)
+          (string? b)]
+   :post [(number? %)]}
+  (let [parse #(mapv parse-int (-> %
+                                   (split #"-")
+                                   (first)
+                                   (split #"[\\._]")))]
+    (compare (parse a) (parse b))))
+
+;; control flow
+
+(defmacro cond-let
+  "Takes a binding-form and a set of test/expr pairs. Evaluates each test
+  one at a time. If a test returns logical true, cond-let evaluates and
+  returns expr with binding-form bound to the value of test and doesn't
+  evaluate any of the other tests or exprs. To provide a default value
+  either provide a literal that evaluates to logical true and is
+  binding-compatible with binding-form, or use :else as the test and don't
+  refer to any parts of binding-form in the expr. (cond-let binding-form)
+  returns nil."
+  [bindings & clauses]
+  (let [binding (first bindings)]
+    (when-let [[test expr & more] clauses]
+      (if (= test :else)
+        expr
+        `(if-let [~binding ~test]
+           ~expr
+           (cond-let ~bindings ~@more))))))
+
+;; Metrics and timing
+
+(defn multitime!*
+  "Helper for `multitime!`. Given a set of timer objects and a
+  function, wrap the function in nested calls to `time!` so that
+  execution of the function has its execution time tracked in each of
+  the supplied timer objects."
+  [timers f]
+  {:pre [(coll? timers)]}
+  (let [wrapped-fn (reduce (fn [thunk timer]
+                             #(time! timer (thunk)))
+                           f
+                           timers)]
+    (wrapped-fn)))
+
+(defmacro multitime!
+  "Like `time!`, but tracks the execution time in each of the supplied
+  timer objects"
+  [timers & body]
+  `(multitime!* ~timers (fn [] (do ~@body))))
